﻿using System.ComponentModel;

namespace GameEditor.Client.Data.DataServices
{
  public partial class GameEntities
  {
    partial void OnContextCreated()
    {
      // Force all game queries to include the Genres and Ratings
      _Games = base.CreateQuery<Game>("Games").Expand("Genre").Expand("Rating");
    }
  }

  public partial class Game : INotifyPropertyChanged
  {
    partial void OnDescriptionChanged() { FirePropertyChanged("Description"); }
    partial void OnDeveloperChanged() { FirePropertyChanged("Developer"); }
    partial void OnGameIDChanged() { FirePropertyChanged("GameID"); }
    partial void OnImageUrlChanged() { FirePropertyChanged("ImageUrl"); }
    partial void OnNameChanged() { FirePropertyChanged("Name"); }
    partial void OnPriceChanged() { FirePropertyChanged("Price"); }
    partial void OnPublisherChanged() { FirePropertyChanged("Publisher"); }
    partial void OnReleaseDateChanged() { FirePropertyChanged("ReleaseDate"); }

    public event PropertyChangedEventHandler PropertyChanged;

    public bool IsValid()
    {
      return (Name.Length > 4 &&
              Price > 0m);
    }

    void FirePropertyChanged(string property)
    {
      if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(property));
    }
  }

  public partial class Genre : INotifyPropertyChanged
  {
    partial void OnGenreIDChanged() { FirePropertyChanged("GenreID"); }
    partial void OnNameChanged() { FirePropertyChanged("Name"); }

    public event PropertyChangedEventHandler PropertyChanged;

    void FirePropertyChanged(string property)
    {
      if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(property));
    }
  }

  public partial class Rating : INotifyPropertyChanged
  {
    partial void OnRatingIDChanged() { FirePropertyChanged("RatingID"); }
    partial void OnNameChanged() { FirePropertyChanged("Name"); }
    partial void OnUrlChanged() { FirePropertyChanged("Url"); }

    public event PropertyChangedEventHandler PropertyChanged;

    void FirePropertyChanged(string property)
    {
      if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(property));
    }
  }
}
