﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Linq;
using System.Diagnostics;

namespace GameEditor.Common.Controls
{
  public class BindableComboBox : System.Windows.Controls.ComboBox
  {
    public BindableComboBox()
    {
      this.Loaded += new RoutedEventHandler(BindableComboBox_Loaded);
      this.SelectionChanged += new SelectionChangedEventHandler(BindableComboBox_SelectionChanged);
    }

    void BindableComboBox_Loaded(object sender, RoutedEventArgs e)
    {
      SetSelectionFromValue();
    }

    private object _selection;

    void BindableComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      if (e.AddedItems.Count > 0)
      {
        _selection = e.AddedItems[0];
        SelectedValue = GetMemberValue(_selection);
      }
      else
      {
        _selection = null;
        SelectedValue = null;
      }
    }

    private object GetMemberValue(object item)
    {
      return item.GetType().GetProperty(ValueMemberPath).GetValue(item, null);
    }

    public static DependencyProperty ValueMemberPathProperty =
      DependencyProperty.Register("ValueMemberPath", typeof(string), typeof(BindableComboBox), null);

    public string ValueMemberPath
    {
      get
      {
        return ((string)(base.GetValue(BindableComboBox.ValueMemberPathProperty)));
      }
      set
      {
        base.SetValue(BindableComboBox.ValueMemberPathProperty, value);
      }
    }

    public static DependencyProperty SelectedValueProperty =
      DependencyProperty.Register("SelectedValue", typeof(object), typeof(BindableComboBox),
      new PropertyMetadata((o, e) =>
      {
        ((BindableComboBox)o).SetSelectionFromValue();
      }));

    public object SelectedValue
    {
      get
      {
        return ((object)(base.GetValue(BindableComboBox.SelectedValueProperty)));
      }
      set
      {
        base.SetValue(BindableComboBox.SelectedValueProperty, value);
      }
    }

    private void SetSelectionFromValue()
    {
      var value = SelectedValue;
      if (Items.Count > 0 && value != null)
      {
        var sel = (from item in this.Items
                   where GetMemberValue(item).Equals(value)
                   select item).SingleOrDefault();
        if (sel == null)
        {
          Debug.WriteLine(string.Concat("Binding Failure, could not set selection in ComboBox.  Value: ", value));
        }
        else
        {
          _selection = sel;
          SelectedItem = sel;
        }
      }
    }

    protected override void OnItemsChanged(System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
    {
      base.OnItemsChanged(e);
      SetSelectionFromValue();
    }
  }
}
