﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Controls.Primitives;
using Microsoft.Practices.Composite.Presentation.Commands;

namespace GameEditor.Common
{
  public class SelectorCommandBehavior : CommandBehaviorBase<Selector>
  {
    public SelectorCommandBehavior(Selector source)
      : base(source)
    {
      source.SelectionChanged += new SelectionChangedEventHandler(source_SelectionChanged);
    }

    void source_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      var theSelector = sender as Selector;

      if (theSelector != null)
      {
        this.Command.Execute(theSelector.SelectedItem);
      }
    }
  }

  public static class SelectionChanged
  {
    private static readonly DependencyProperty SelectionChangedCommandBehaviorProperty = DependencyProperty.RegisterAttached(
        "SelectionChangedCommandBehavior",
        typeof(SelectorCommandBehavior),
        typeof(SelectionChanged),
        null);


    public static readonly DependencyProperty CommandProperty = DependencyProperty.RegisterAttached(
        "Command",
        typeof(ICommand),
        typeof(SelectionChanged),
        new PropertyMetadata(OnSetCommandCallback));

    public static readonly DependencyProperty CommandParameterProperty = DependencyProperty.RegisterAttached(
        "CommandParameter",
        typeof(object),
        typeof(SelectionChanged),
        new PropertyMetadata(OnSetCommandParameterCallback));


    public static void SetCommand(Selector theSelector, ICommand command)
    {
      theSelector.SetValue(CommandProperty, command);
    }

    public static ICommand GetCommand(Selector theSelector)
    {
      return theSelector.GetValue(CommandProperty) as ICommand;
    }

    public static void SetCommandParameter(Selector theSelector, object parameter)
    {
      theSelector.SetValue(CommandParameterProperty, parameter);
    }

    public static object GetCommandParameter(Selector theSelector)
    {
      return theSelector.GetValue(CommandParameterProperty);
    }

    private static void OnSetCommandCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
    {
      Selector theSelector = dependencyObject as Selector;
      if (theSelector != null)
      {
        SelectorCommandBehavior behavior = GetOrCreateBehavior(theSelector);
        behavior.Command = e.NewValue as ICommand;
      }
    }

    private static void OnSetCommandParameterCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
    {
      Selector theSelector = dependencyObject as Selector;
      if (theSelector != null)
      {
        SelectorCommandBehavior behavior = GetOrCreateBehavior(theSelector);
        behavior.CommandParameter = e.NewValue;
      }
    }

    private static SelectorCommandBehavior GetOrCreateBehavior(Selector theSelector)
    {
      SelectorCommandBehavior behavior = theSelector.GetValue(SelectionChangedCommandBehaviorProperty) as SelectorCommandBehavior;
      if (behavior == null)
      {
        behavior = new SelectorCommandBehavior(theSelector);
        theSelector.SetValue(SelectionChangedCommandBehaviorProperty, behavior);
      }

      return behavior;
    }
  }
}
