﻿using System;
using System.Linq;
using System.Threading;
using System.Windows;
using GameEditor.Client.Data;
using GameEditor.Client.Data.DataServices;
using GameEditor.Client.Data.EventArgs;
using GameEditor.Client.Data.Events;
using GameEditor.Common;
using Microsoft.Practices.Composite.Events;

namespace GameEditor.GameDetailsHost
{
  public class MockGameModel : IGameModel
  {
    Genre[] genres = new Genre[] 
    { 
      new Genre() { GenreID = 1, Name = "Shooter" },
      new Genre() { GenreID = 2, Name = "Family" }
    };

    Rating[] ratings = new Rating[] 
    { 
      new Rating() { RatingID = 1, Name = "E (Everybody)" },
      new Rating() { RatingID = 2, Name = "M (Mature)" }
    };
    IEventAggregator theEventAggregator;

    public MockGameModel(IEventAggregator eventAggregator)
    {
      theEventAggregator = eventAggregator;
      ThreadPool.QueueUserWorkItem(new WaitCallback(OnStartup));
    }

    void OnStartup(object ignore)
    {
      Deployment.Current.Dispatcher.BeginInvoke(
          () =>
          {
            Thread.Sleep(1000);
            Game mockGame = new Game()
            {
              GameID = 1,
              Name = "My Fake Game",
              Price = 59.99m,
              Publisher = "Microsoft Games",
              Developer = "Microsoft Games",
              Description = "This is some game.",
              Genre = genres[0],
              Rating = ratings[0],
              ReleaseDate = DateTime.Today.AddYears(-2),
              ImageUrl = "http://www.xbox.com/NR/rdonlyres/6BE895B2-AB98-423A-BA64-E9A993C953F8/0/boxApprovedGenericXbox.jpg"
            };
            theEventAggregator.GetEvent<GameSelectedEvent>().Publish(mockGame);
          }
        );
    }

    public void LoadGamesAsync()
    {
      throw new NotImplementedException();
    }

    public void LoadGamesByGenreAsync(string genre)
    {
      throw new NotImplementedException();
    }

    public event EventHandler<LoadEntityCompleteEventArgs<Game>> LoadGamesComplete;

    public void LoadGenresAsync()
    {
      App.Current.RootVisual.Dispatcher.BeginInvoke(
          () =>
          {
            if (LoadGenresComplete != null)
            {
              LoadGenresComplete(this,
                new LoadEntityCompleteEventArgs<Genre>(
                  genres.AsEnumerable<Genre>()));
            }
          }
        );
    }

    public event EventHandler<LoadEntityCompleteEventArgs<Genre>> LoadGenresComplete;

    public void LoadRatingsAsync()
    {
      App.Current.RootVisual.Dispatcher.BeginInvoke(
          () =>
          {
            if (LoadRatingsComplete != null)
            {
              LoadRatingsComplete(this, new LoadEntityCompleteEventArgs<Rating>(
                  ratings.AsEnumerable<Rating>()));
            }
          }
        );
    }

    public event EventHandler<LoadEntityCompleteEventArgs<Rating>> LoadRatingsComplete;

    public void SaveAsync()
    {
      ThreadPool.QueueUserWorkItem(new WaitCallback(OnSave));
    }

    void OnSave(object o)
    {
      Thread.Sleep(2000);
      Deployment.Current.Dispatcher.BeginInvoke(
        () =>
        {
          if (SaveComplete != null)
          {
            SaveComplete(this, new ErrorEventArgs());
          }
        }
        );
    }

    public event EventHandler<ErrorEventArgs> SaveComplete;
  }
}
