﻿using System;
using System.Windows;
using GameEditor.Common.Interfaces;
using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Composite.UnityExtensions;

namespace GameEditor
{
  public class Bootstrapper : UnityBootstrapper
  {
    protected override void ConfigureContainer()
    {
      Container.RegisterType<IShellProvider, Shell>();
      base.ConfigureContainer();
    }

    protected override DependencyObject CreateShell()
    {
      // Get the provider for the shell
      IShellProvider shellProvider = Container.Resolve<IShellProvider>();

      // Tell the provider to create the shell
      UIElement theShell = shellProvider.CreateShell();

      // Assign the shell to the root visual of our App
      App.Current.RootVisual = theShell;

      // Return the Shell
      return theShell;
    }

    protected override IModuleCatalog GetModuleCatalog()
    {
      var catalog = ModuleCatalog.CreateFromXaml(new Uri("catalog.xaml", UriKind.Relative));
      return catalog;
    }
  }
}
