Imports Microsoft.VisualBasic
Imports System
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Ink
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports System.Linq
Imports System.Diagnostics
Imports System.Collections.Specialized

Namespace Controls
  Public Class BindableComboBox
    Inherits System.Windows.Controls.ComboBox
    Public Sub New()
      AddHandler Loaded, AddressOf BindableComboBox_Loaded
      AddHandler SelectionChanged, AddressOf BindableComboBox_SelectionChanged
    End Sub

    Private Sub BindableComboBox_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
      SetSelectionFromValue()
    End Sub

    Private _selection As Object

    Private Sub BindableComboBox_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
      If e.AddedItems.Count > 0 Then
        _selection = e.AddedItems(0)
        SelectedValue = GetMemberValue(_selection)
      Else
        _selection = Nothing
        SelectedValue = Nothing
      End If
    End Sub

    Private Function GetMemberValue(ByVal item As Object) As Object
      Return item.GetType().GetProperty(ValueMemberPath).GetValue(item, Nothing)
    End Function

    Public Shared ValueMemberPathProperty As DependencyProperty = _
      DependencyProperty.Register("ValueMemberPath", _
                                  GetType(String), _
                                  GetType(BindableComboBox), Nothing)

    Public Property ValueMemberPath() As String
      Get
        Return (CStr(MyBase.GetValue(BindableComboBox.ValueMemberPathProperty)))
      End Get
      Set(ByVal value As String)
        MyBase.SetValue(BindableComboBox.ValueMemberPathProperty, value)
      End Set
    End Property

    Public Shared SelectedValueProperty As DependencyProperty _
      = DependencyProperty.Register("SelectedValue", _
                                    GetType(Object), _
                                    GetType(BindableComboBox), _
                                    New PropertyMetadata(AddressOf OnSelectedValue))

    Private Shared Sub OnSelectedValue(ByVal o As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
      CType(o, BindableComboBox).SetSelectionFromValue()
    End Sub

    Public Property SelectedValue() As Object
      Get
        Return (CObj(MyBase.GetValue(BindableComboBox.SelectedValueProperty)))
      End Get
      Set(ByVal value As Object)
        MyBase.SetValue(BindableComboBox.SelectedValueProperty, value)
      End Set
    End Property

    Private Sub SetSelectionFromValue()
      Dim value = SelectedValue
      If Items.Count > 0 AndAlso value IsNot Nothing Then
        Dim sel = (From item In Me.Items _
                   Where GetMemberValue(item).Equals(value) _
                   Select item).SingleOrDefault()
        If sel Is Nothing Then
          Debug.WriteLine(String.Concat("Binding Failure, could not set selection in ComboBox.  Value: ", value))
        Else
          _selection = sel
          SelectedItem = sel
        End If
      End If
    End Sub

    Protected Overrides Sub OnItemsChanged(ByVal e As NotifyCollectionChangedEventArgs)
      MyBase.OnItemsChanged(e)
      SetSelectionFromValue()
    End Sub
  End Class
End Namespace
