Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Controls.Primitives
Imports Microsoft.Practices.Composite.Presentation.Commands
Imports System.Windows.Input

Public Class SelectorCommandBehavior
  Inherits CommandBehaviorBase(Of Selector)
  Public Sub New(ByVal source As Selector)
    MyBase.New(source)
    AddHandler source.SelectionChanged, AddressOf source_SelectionChanged
  End Sub

  Private Sub source_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
    Dim theSelector = TryCast(sender, Selector)

    If theSelector IsNot Nothing Then
      Me.Command.Execute(theSelector.SelectedItem)
    End If
  End Sub
End Class

Public NotInheritable Class SelectionChanged
  Private Shared ReadOnly SelectionChangedCommandBehaviorProperty As DependencyProperty = _
    DependencyProperty.RegisterAttached("SelectionChangedCommandBehavior", _
                                        GetType(SelectorCommandBehavior), _
                                        GetType(SelectionChanged), Nothing)

  Public Shared ReadOnly CommandProperty As DependencyProperty = _
    DependencyProperty.RegisterAttached("Command", _
                                        GetType(ICommand), _
                                        GetType(SelectionChanged), _
                                        New PropertyMetadata(AddressOf OnSetCommandCallback))

  Public Shared ReadOnly CommandParameterProperty As DependencyProperty = _
    DependencyProperty.RegisterAttached("CommandParameter", _
                                        GetType(Object), _
                                        GetType(SelectionChanged), _
                                        New PropertyMetadata(AddressOf OnSetCommandParameterCallback))


  Private Sub New()
  End Sub

  Public Shared Sub SetCommand(ByVal theSelector As Selector, ByVal command As ICommand)
    theSelector.SetValue(CommandProperty, command)
  End Sub

  Public Shared Function GetCommand(ByVal theSelector As Selector) As ICommand
    Return TryCast(theSelector.GetValue(CommandProperty), ICommand)
  End Function

  Public Shared Sub SetCommandParameter(ByVal theSelector As Selector, ByVal parameter As Object)
    theSelector.SetValue(CommandParameterProperty, parameter)
  End Sub

  Public Shared Function GetCommandParameter(ByVal theSelector As Selector) As Object
    Return theSelector.GetValue(CommandParameterProperty)
  End Function

  Private Shared Sub OnSetCommandCallback(ByVal dependencyObject As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
    Dim theSelector As Selector = TryCast(dependencyObject, Selector)
    If theSelector IsNot Nothing Then
      Dim behavior As SelectorCommandBehavior = GetOrCreateBehavior(theSelector)
      behavior.Command = TryCast(e.NewValue, ICommand)
    End If
  End Sub

  Private Shared Sub OnSetCommandParameterCallback(ByVal dependencyObject As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
    Dim theSelector As Selector = TryCast(dependencyObject, Selector)
    If theSelector IsNot Nothing Then
      Dim behavior As SelectorCommandBehavior = GetOrCreateBehavior(theSelector)
      behavior.CommandParameter = e.NewValue
    End If
  End Sub

  Private Shared Function GetOrCreateBehavior(ByVal theSelector As Selector) As SelectorCommandBehavior
    Dim behavior As SelectorCommandBehavior = TryCast(theSelector.GetValue(SelectionChangedCommandBehaviorProperty), SelectorCommandBehavior)
    If behavior Is Nothing Then
      behavior = New SelectorCommandBehavior(theSelector)
      theSelector.SetValue(SelectionChangedCommandBehaviorProperty, behavior)
    End If

    Return behavior
  End Function
End Class
