Imports Microsoft.VisualBasic
Imports System
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Ink
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports GameEditor.Client.Data
Imports Microsoft.Practices.Unity
Imports System.Collections.ObjectModel
Imports GameEditor.Client.Data.DataServices
Imports System.ComponentModel
Imports System.Collections.Generic
Imports Microsoft.Practices.Composite.Presentation.Commands
Imports Microsoft.Practices.Composite.Events
Imports GameEditor.Common.Interfaces
Imports GameEditor.Common
Imports GameEditor.Client.Data.EventArgs
Imports GameEditor.Client.Data.Events
Imports Microsoft.Practices.Composite.Modularity

Namespace ViewModels
  Public Class GameListViewModel
    Inherits ViewModelBase
    Implements IGameListViewModel

    Private theModel As IGameModel = Nothing
    Private theEventAggregator As IEventAggregator = Nothing
    Private theModuleManager As IModuleManager = Nothing
    Private [error] As String = Nothing
    Private isLoading_Renamed As Boolean = False

    Public Sub New(ByVal model As IGameModel, ByVal eventAggregator As IEventAggregator, ByVal modMgr As IModuleManager)
      ' Keep a reference to injected services
      theEventAggregator = eventAggregator
      theModel = model
      theModuleManager = modMgr

      ' Initialize the bindable collection
      Games = New ObservableCollection(Of Game)()

      ' Setup the Commands
      LoadCommand = New DelegateCommand(Of Object)(AddressOf LoadAsync)
      SelectionChanged = New DelegateCommand(Of Object)(AddressOf HandleSelectionChanged)

      ' Wire up Events
      AddHandler theModel.LoadGamesComplete, AddressOf theModel_LoadGamesComplete

    End Sub

    Private Sub HandleSelectionChanged(ByVal o As Object)
      ' Fire Selection Changed with Global Event
      theEventAggregator.GetEvent(Of GameSelectedEvent)().Publish(TryCast(o, Game))
    End Sub

    Public Sub LoadAsync()
      IsLoading = True
      theModel.LoadGamesAsync()
    End Sub

    Public Sub LoadByGenreAsync(ByVal genre As String)
      IsLoading = True
      theModel.LoadGamesByGenreAsync(genre)
    End Sub

    Private Sub theModel_LoadGamesComplete(ByVal sender As Object, ByVal e As LoadEntityCompleteEventArgs(Of Game))
      If e.Error IsNot Nothing Then
        LoadError = String.Concat("Load Failed: ", e.Error.Message)
      Else
        Games.Clear()

        For Each game In e.Results
          Games.Add(game)
        Next game

        RaiseEvent CollectionLoaded(Me, New EventArgs())
      End If

      ' Since we now have games, let's load the detail pane
      theModuleManager.LoadModule("GameEditor.GameDetails")

      IsLoading = False
    End Sub

    Public Event CollectionLoaded As EventHandler

    Private privateLoadCommand As DelegateCommand(Of Object)
    Public Property LoadCommand() As DelegateCommand(Of Object)
      Get
        Return privateLoadCommand
      End Get
      Protected Set(ByVal value As DelegateCommand(Of Object))
        privateLoadCommand = value
      End Set
    End Property
    Private privateSelectionChanged As DelegateCommand(Of Object)
    Public Property SelectionChanged() As DelegateCommand(Of Object)
      Get
        Return privateSelectionChanged
      End Get
      Protected Set(ByVal value As DelegateCommand(Of Object))
        privateSelectionChanged = value
      End Set
    End Property

    Public Property LoadError() As String
      Get
        Return [error]
      End Get
      Private Set(ByVal value As String)
        If [error] <> value Then
          [error] = value
          FirePropertyChanged("LoadError")
        End If
      End Set
    End Property

    Public Property IsLoading() As Boolean
      Get
        Return isLoading_Renamed
      End Get
      Private Set(ByVal value As Boolean)
        If isLoading_Renamed <> value Then
          isLoading_Renamed = value
          FirePropertyChanged("IsLoading")
        End If
      End Set
    End Property

    Private privateGames As ObservableCollection(Of Game)
    Public Property Games() As ObservableCollection(Of Game)
      Get
        Return privateGames
      End Get
      Protected Set(ByVal value As ObservableCollection(Of Game))
        privateGames = value
      End Set
    End Property

    Public Property Model() As IModel Implements IGameListViewModel.Model
      Get
        Return theModel
      End Get
      Set(ByVal value As IModel)
        theModel = TryCast(value, IGameModel)
      End Set
    End Property

  End Class
End Namespace
