﻿Imports System
Imports System.Linq
Imports System.Collections.Generic
Imports System.Threading
Imports System.Windows
Imports GameEditor.Client.Data
Imports GameEditor.Client.Data.DataServices
Imports GameEditor.Client.Data.EventArgs
Imports GameEditor.Client.Data.Events
Imports GameEditor.Common
Imports Microsoft.Practices.Composite.Events

Public Class MockGameModel
  Implements IGameModel

  Private genres(1) As Genre
  Private ratings(1) As Rating
  Private theEventAggregator As IEventAggregator

  Public Sub New(ByVal eventAggregator As IEventAggregator)

    genres(0) = New Genre() With {.GenreID = 1, .Name = "Shooter"}
    genres(1) = New Genre() With {.GenreID = 2, .Name = "Family"}
    ratings(0) = New Rating() With {.RatingID = 1, .Name = "E (Everybody)"}
    ratings(1) = New Rating() With {.RatingID = 2, .Name = "M (Mature)"}

    theEventAggregator = eventAggregator
    ThreadPool.QueueUserWorkItem(New WaitCallback(AddressOf OnStartup))

  End Sub

  Private Sub OnStartup(ByVal ignore As Object)

    Deployment.Current.Dispatcher.BeginInvoke(AddressOf OnStartupMethod)

  End Sub

  Sub OnStartupMethod()

    Thread.Sleep(1000)

    Dim mockGame As New Game() With {.GameID = 1, .Name = "My Fake Game", _
                                     .Price = 59.99D, .Publisher = "Microsoft Games", _
                                     .Developer = "Microsoft Games", _
                                     .Description = "This is some game.", _
                                     .Genre = genres(0), _
                                     .Rating = ratings(0), _
                                     .ReleaseDate = DateTime.Today.AddYears(-2), _
                                     .ImageUrl = "http://www.xbox.com/NR/rdonlyres/6BE895B2-AB98-423A-BA64-E9A993C953F8/0/boxApprovedGenericXbox.jpg"}

    theEventAggregator.GetEvent(Of GameSelectedEvent)().Publish(mockGame)

  End Sub

  Public Sub LoadGamesAsync() Implements IGameModel.LoadGamesAsync
    Throw New NotImplementedException()
  End Sub

  Public Sub LoadGamesByGenreAsync(ByVal genre As String) Implements IGameModel.LoadGamesByGenreAsync
    Throw New NotImplementedException()
  End Sub

  Public Event LoadGamesComplete As EventHandler(Of LoadEntityCompleteEventArgs(Of Game)) Implements IGameModel.LoadGamesComplete

  Public Sub LoadGenresAsync() Implements IGameModel.LoadGenresAsync
    App.Current.RootVisual.Dispatcher.BeginInvoke(AddressOf OnLoadGenresMethod)
  End Sub

  Sub OnLoadGenresMethod()
    RaiseEvent LoadGenresComplete(Me, New LoadEntityCompleteEventArgs(Of Genre)(genres.AsEnumerable()))
  End Sub

  Public Event LoadGenresComplete As EventHandler(Of LoadEntityCompleteEventArgs(Of Genre)) Implements IGameModel.LoadGenresComplete

  Public Sub LoadRatingsAsync() Implements IGameModel.LoadRatingsAsync
    App.Current.RootVisual.Dispatcher.BeginInvoke(AddressOf OnLoadRatingsMethod)
  End Sub

  Sub OnLoadRatingsMethod()
    RaiseEvent LoadRatingsComplete(Me, New LoadEntityCompleteEventArgs(Of Rating)(ratings.AsEnumerable()))
  End Sub

  Public Event LoadRatingsComplete As EventHandler(Of LoadEntityCompleteEventArgs(Of Rating)) Implements IGameModel.LoadRatingsComplete

  Public Sub SaveAsync() Implements IGameModel.SaveAsync
    ThreadPool.QueueUserWorkItem(New WaitCallback(AddressOf OnSave))
  End Sub

  Private Sub OnSave(ByVal o As Object)

    Thread.Sleep(2000)
    Deployment.Current.Dispatcher.BeginInvoke(AddressOf OnSaveMethod)

  End Sub

  Sub OnSaveMethod()
    RaiseEvent SaveComplete(Me, New ErrorEventArgs())
  End Sub

  Public Event SaveComplete As EventHandler(Of ErrorEventArgs) Implements IGameModel.SaveComplete

End Class